@@ -240,16 +240,56 @@ class security_manager {
     }
     
     public function block_ip($ip_address, $reason = 'Manual block', $failed_attempts = 0, $permanent = false, $skip_service_blocks = false) {
-        // Don't block whitelisted IPs
-        if ($this->is_ip_whitelisted($ip_address)) {
+        // Validate IP or CIDR format
+        if (!$this->validate_ip_or_cidr($ip_address)) {
+            error_log("Invalid IP or CIDR format: " . $ip_address);
             return false;
         }
-        
-        // Check if IP is already blocked in security manager to prevent duplicates
-        if ($this->is_ip_blocked($ip_address)) {
+
+        // Normalize CIDR notation to canonical form
+        $ip_address = $this->normalize_cidr($ip_address);
+        if ($ip_address === false) {
+            error_log("Failed to normalize IP/CIDR: " . $ip_address);
+            return false;
+        }
+
+        // Check if this IP/CIDR is whitelisted or overlaps with exempted ranges
+        if ($this->is_ip_whitelisted($ip_address)) {
+            error_log("Cannot block whitelisted IP/CIDR: " . $ip_address);
+            return false;
+        }
+
+        // For CIDR ranges, check if they overlap with any exempted ranges
+        if (strpos($ip_address, '/') !== false) {
+            // Limit number of CIDR ranges to prevent DoS
+            $stmt = $this->db->prepare("SELECT COUNT(*) as count FROM blocked_ips WHERE ip_address LIKE '%/%'");
+            $result = $stmt->execute();
+            $row = $result->fetchArray(SQLITE3_ASSOC);
+
+            if ($row['count'] >= 1000) {
+                error_log("CIDR range limit exceeded: {$row['count']}/1000");
+                return false;
+            }
+
+            // Check if this CIDR range overlaps with any exempted range
+            $exemption_ips = $this->get_exemption_ips();
+            foreach ($exemption_ips as $exempt) {
+                if ($this->cidr_ranges_overlap($ip_address, $exempt['ip_address'])) {
+                    error_log("Cannot block CIDR range that overlaps with exempted range: " . $exempt['ip_address']);
+                    return false;
+                }
+            }
+        }
+
+        // Check if IP/CIDR is already blocked in security manager to prevent duplicates
+        // For CIDR, check exact match only
+        $stmt = $this->db->prepare('SELECT * FROM blocked_ips WHERE ip_address = :ip');
+        $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
+        $result = $stmt->execute();
+        if ($result->fetchArray(SQLITE3_ASSOC)) {
             return true; // Already blocked, return success
         }
-        
+
         // Store in local database
         $unblock_time = $permanent ? -1 : (time() + $this->block_duration); // -1 means permanent/manual unblock only
         $stmt = $this->db->prepare('INSERT OR REPLACE INTO blocked_ips (ip_address, block_time, unblock_time, reason, failed_attempts, manual_block) VALUES (:ip, :block_time, :unblock_time, :reason, :attempts, :manual)');
@@ -260,7 +300,7 @@ class security_manager {
         $stmt->bindValue(':attempts', $failed_attempts, SQLITE3_INTEGER);
         $stmt->bindValue(':manual', ($reason === 'Manual block') ? 1 : 0, SQLITE3_INTEGER);
         $stmt->execute();
-        
+
         // Also block in HAProxy for full service-wide blocking
         try {
             require_once('/docker/whp/web/libs/haproxy_manager.php');
@@ -270,9 +310,10 @@ class security_manager {
             error_log("Failed to block IP in HAProxy: " . $e->getMessage());
             // Continue even if HAProxy blocking fails - local block is still active
         }
-        
+
         // Also block in SSH/FTP services via log monitor (only if not called from service blocks to prevent loops)
-        if (!$skip_service_blocks) {
+        // Note: SSH/FTP services may not support CIDR directly, so skip for CIDR ranges
+        if (!$skip_service_blocks && strpos($ip_address, '/') === false) {
             try {
                 require_once('/docker/whp/web/libs/log_monitor_manager.php');
                 $log_monitor = new log_monitor_manager();
@@ -284,7 +325,7 @@ class security_manager {
                 // Continue even if service blocking fails
             }
         }
-        
+
         return true;
     }
     
@@ -329,48 +370,226 @@ class security_manager {
         if ($this->is_ip_whitelisted($ip_address)) {
             return false;
         }
-        
-        // Check if IP is blocked
-        $stmt = $this->db->prepare('SELECT * FROM blocked_ips WHERE ip_address = :ip AND (unblock_time > :time OR unblock_time = -1)');
-        $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
+
+        // Check if IP is blocked (including CIDR ranges)
+        $stmt = $this->db->prepare('SELECT * FROM blocked_ips WHERE (unblock_time > :time OR unblock_time = -1)');
         $stmt->bindValue(':time', time(), SQLITE3_INTEGER);
         $result = $stmt->execute();
-        $row = $result->fetchArray(SQLITE3_ASSOC);
-        
-        if ($row) {
-            // Auto-unblock if time has expired (but not for permanent blocks with unblock_time = -1)
-            if ($row['unblock_time'] > 0 && $row['unblock_time'] <= time()) {
-                $this->unblock_ip($ip_address);
-                return false;
+
+        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
+            $blocked_entry = $row['ip_address'];
+
+            // Check if IP matches this blocked entry (handles both single IPs and CIDR)
+            if ($this->ip_in_cidr($ip_address, $blocked_entry)) {
+                // Auto-unblock if time has expired (but not for permanent blocks with unblock_time = -1)
+                if ($row['unblock_time'] > 0 && $row['unblock_time'] <= time()) {
+                    $this->unblock_ip($blocked_entry);
+                    return false;
+                }
+                return $row;
             }
-            return $row;
         }
-        
+
         return false;
     }
     
+    /**
+     * Validates if a string is a valid IP address or CIDR notation
+     *
+     * @param string $input IP address or CIDR notation (e.g., "192.168.1.0/24")
+     * @return bool True if valid IP or CIDR, false otherwise
+     */
+    private function validate_ip_or_cidr($input) {
+        // Check if it's a plain IP address
+        if (filter_var($input, FILTER_VALIDATE_IP)) {
+            return true;
+        }
+
+        // Check if it's CIDR notation
+        if (strpos($input, '/') !== false) {
+            list($ip, $mask) = explode('/', $input, 2);
+
+            // Validate IP part
+            if (!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
+                return false;
+            }
+
+            // Validate mask (0-32 for IPv4)
+            if (!is_numeric($mask) || $mask < 0 || $mask > 32) {
+                return false;
+            }
+
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Checks if an IP address matches a CIDR range
+     *
+     * @param string $ip IP address to check
+     * @param string $cidr CIDR notation (e.g., "192.168.1.0/24")
+     * @return bool True if IP is in CIDR range, false otherwise
+     */
+    private function ip_in_cidr($ip, $cidr) {
+        // If CIDR doesn't contain /, it's just a single IP
+        if (strpos($cidr, '/') === false) {
+            return $ip === $cidr;
+        }
+
+        list($subnet, $mask) = explode('/', $cidr, 2);
+
+        // Validate mask value
+        $mask = (int)$mask;
+        if ($mask < 0 || $mask > 32) {
+            error_log("Invalid CIDR mask value: $mask");
+            return false;
+        }
+
+        // Convert IP and subnet to long integers
+        $ip_long = ip2long($ip);
+        $subnet_long = ip2long($subnet);
+
+        if ($ip_long === false || $subnet_long === false) {
+            return false;
+        }
+
+        // Handle edge cases to prevent integer overflow
+        if ($mask === 0) {
+            // /0 matches all IPs
+            return true;
+        }
+
+        if ($mask === 32) {
+            // /32 is a single IP
+            return $ip_long === $subnet_long;
+        }
+
+        // Calculate subnet mask with overflow protection
+        $mask_long = (0xFFFFFFFF << (32 - $mask)) & 0xFFFFFFFF;
+
+        // Apply mask to both IP and subnet and compare
+        return ($ip_long & $mask_long) === ($subnet_long & $mask_long);
+    }
+
+    /**
+     * Normalize CIDR notation to canonical form
+     *
+     * @param string $cidr CIDR notation or IP address
+     * @return string|false Normalized CIDR or false on error
+     */
+    private function normalize_cidr($cidr) {
+        if (strpos($cidr, '/') === false) {
+            return $cidr; // Single IP, no normalization needed
+        }
+
+        list($ip, $mask) = explode('/', $cidr, 2);
+        $mask = (int)$mask;
+
+        // Calculate network address
+        $ip_long = ip2long($ip);
+        if ($ip_long === false) {
+            return false;
+        }
+
+        if ($mask === 0) {
+            $network_long = 0;
+        } else if ($mask === 32) {
+            $network_long = $ip_long;
+        } else {
+            $mask_long = (0xFFFFFFFF << (32 - $mask)) & 0xFFFFFFFF;
+            $network_long = $ip_long & $mask_long;
+        }
+
+        $network_ip = long2ip($network_long);
+        return $network_ip . '/' . $mask;
+    }
+
+    /**
+     * Check if two CIDR ranges overlap
+     *
+     * @param string $cidr1 First CIDR range or IP
+     * @param string $cidr2 Second CIDR range or IP
+     * @return bool True if ranges overlap, false otherwise
+     */
+    private function cidr_ranges_overlap($cidr1, $cidr2) {
+        $cidr1_has_mask = strpos($cidr1, '/') !== false;
+        $cidr2_has_mask = strpos($cidr2, '/') !== false;
+
+        // If both are single IPs, just compare
+        if (!$cidr1_has_mask && !$cidr2_has_mask) {
+            return $cidr1 === $cidr2;
+        }
+
+        // If one is single IP, check if it's in the other's range
+        if (!$cidr1_has_mask) {
+            return $this->ip_in_cidr($cidr1, $cidr2);
+        }
+        if (!$cidr2_has_mask) {
+            return $this->ip_in_cidr($cidr2, $cidr1);
+        }
+
+        // Both are CIDR ranges - check if they overlap
+        list($ip1, $mask1) = explode('/', $cidr1);
+        list($ip2, $mask2) = explode('/', $cidr2);
+
+        $ip1_long = ip2long($ip1);
+        $ip2_long = ip2long($ip2);
+
+        if ($ip1_long === false || $ip2_long === false) {
+            return false;
+        }
+
+        $mask1 = (int)$mask1;
+        $mask2 = (int)$mask2;
+
+        $mask1_long = $mask1 === 0 ? 0 : ((0xFFFFFFFF << (32 - $mask1)) & 0xFFFFFFFF);
+        $mask2_long = $mask2 === 0 ? 0 : ((0xFFFFFFFF << (32 - $mask2)) & 0xFFFFFFFF);
+
+        $network1 = $ip1_long & $mask1_long;
+        $network2 = $ip2_long & $mask2_long;
+
+        // Check if one range contains the other's network address
+        return ($network1 & $mask2_long) === $network2 ||
+               ($network2 & $mask1_long) === $network1;
+    }
+
     public function is_ip_whitelisted($ip_address) {
         // Check built-in whitelist
         if (in_array($ip_address, $this->whitelist_ips)) {
             return true;
         }
-        
-        // Check database whitelist
-        $stmt = $this->db->prepare('SELECT * FROM whitelist_ips WHERE ip_address = :ip');
-        $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
+
+        // Check database whitelist (including CIDR ranges)
+        $stmt = $this->db->prepare('SELECT ip_address FROM whitelist_ips');
         $result = $stmt->execute();
-        $row = $result->fetchArray(SQLITE3_ASSOC);
-        
-        return $row !== false;
+
+        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
+            $whitelist_entry = $row['ip_address'];
+
+            // Check if IP matches this whitelist entry (handles both single IPs and CIDR)
+            if ($this->ip_in_cidr($ip_address, $whitelist_entry)) {
+                return true;
+            }
+        }
+
+        return false;
     }
     
     public function add_whitelist_ip($ip_address, $description = '') {
+        // Validate IP or CIDR format
+        if (!$this->validate_ip_or_cidr($ip_address)) {
+            error_log("Invalid IP or CIDR format: " . $ip_address);
+            return false;
+        }
+
         $stmt = $this->db->prepare('INSERT OR REPLACE INTO whitelist_ips (ip_address, description, added_time) VALUES (:ip, :desc, :time)');
         $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
         $stmt->bindValue(':desc', $description, SQLITE3_TEXT);
         $stmt->bindValue(':time', time(), SQLITE3_INTEGER);
         $stmt->execute();
-        
+
         return true;
     }
     
